#include "paul.h"

const float pi = 3.14159265359f;
static bool keyPress = false;

//=============================================================================
// default constructor
//=============================================================================
Paul::Paul() : Entity()
{
    spriteData.width = paulNS::WIDTH;           // size of paul
    spriteData.height = paulNS::HEIGHT;
    spriteData.x = paulNS::X;                   // location on screen
    spriteData.y = paulNS::Y;
    spriteData.rect.bottom = paulNS::HEIGHT;    // rectangle to select parts of an image
    spriteData.rect.right = paulNS::WIDTH;
    velocity.x = 0;                             // velocity X
    velocity.y = 0;                             // velocity Y
    frameDelay = paulNS::PAUL_ANIMATION_DELAY;
    startFrame = paulNS::PAUL_STAND_FRAME_S;      // PAUL starts at frame 149
    endFrame     = paulNS::PAUL_STAND_FRAME_S;      // PAUL starts at frame 149
    currentFrame = startFrame;
    radius = paulNS::WIDTH/2.0;
    mass = paulNS::MASS;
	explosionOn = false;
	collisionType = entityNS::CIRCLE;
	shieldOn = false;
	paulshotDir = 6;
	shotDir = 6;
	loop = true;
}

//=============================================================================
// Initialize the paul.
// Post: returns true if successful, false if failed
//=============================================================================
bool Paul::initialize(Game *gamePtr, int width, int height, int ncols,
    TextureManager *textureM)
{
	explosion.initialize(gamePtr->getGraphics(), width, height, ncols, textureM);
    explosion.setFrames(paulNS::EXPLOSION_START_FRAME, paulNS::EXPLOSION_END_FRAME);
    explosion.setCurrentFrame(paulNS::EXPLOSION_START_FRAME);
    explosion.setFrameDelay(paulNS::EXPLOSION_ANIMATION_DELAY);
    explosion.setLoop(false);               // do not loop animation
	shield.initialize(gamePtr->getGraphics(), width, height, ncols, textureM);
    shield.setFrames(paulNS::SHIELD_START_FRAME, paulNS::SHIELD_END_FRAME);
    shield.setCurrentFrame(paulNS::SHIELD_START_FRAME);
    shield.setFrameDelay(paulNS::SHIELD_ANIMATION_DELAY);
    shield.setLoop(false);                  // do not loop animation
    return(Entity::initialize(gamePtr, width, height, ncols, textureM));
}

//=============================================================================
// draw paul
//=============================================================================
void Paul::draw()
{
	if(explosionOn)
		explosion.draw(spriteData);
	else
	{
    Image::draw();              // draw paul
	if(shieldOn)
		shield.draw(spriteData, graphicsNS::ALPHA50 & colorFilter);
	}
}

//=============================================================================
// update
// typically called once per frame
// frameTime is used to regulate the speed of movement and animation
//=============================================================================
void Paul::update(float frameTime)
{
	if(explosionOn)
    {
        explosion.update(frameTime);
        if(explosion.getAnimationComplete())    // if explosion animation complete
        {
            explosionOn = false;                // turn off explosion
            visible = false;
            explosion.setAnimationComplete(false);
            explosion.setCurrentFrame(paulNS::EXPLOSION_START_FRAME);
        }
    }

	if(shieldOn)
	{
		shield.update(frameTime);
		if(shield.getAnimationComplete())
		{
			shieldOn = false;
			shield.setAnimationComplete(false);
			shield.setCurrentFrame(paulNS::SHIELD_START_FRAME);
		}
	}

	keyPress = input->anyKeyPressed();

	if(keyPress == false)
	{
		switch (shotDir)
		{
			case 12:
				setFiring(false);
				startFrame = paulNS::PAUL_STAND_FRAME_N;
				endFrame = paulNS::PAUL_STAND_FRAME_N;
				//currentFrame = paulNS::PAUL_STAND_FRAME_N;
			break;
			case 3:
				startFrame = paulNS::PAUL_STAND_FRAME_E;
				endFrame = paulNS::PAUL_STAND_FRAME_E;
				//currentFrame = paulNS::PAUL_STAND_FRAME_E;
			break;
			case 6:
				startFrame = paulNS::PAUL_STAND_FRAME_S;
				endFrame = paulNS::PAUL_STAND_FRAME_S;
				//currentFrame = paulNS::PAUL_STAND_FRAME_S;
			break;
			case 9:
				setFiring(false);
				startFrame = paulNS::PAUL_STAND_FRAME_W;
				endFrame = paulNS::PAUL_STAND_FRAME_W;
				//currentFrame = paulNS::PAUL_STAND_FRAME_W;
			break;
			case 1:
				setFiring(false);
				startFrame = paulNS::PAUL_STAND_FRAME_NE;
				endFrame = paulNS::PAUL_STAND_FRAME_NE;
				//currentFrame = paulNS::PAUL_STAND_FRAME_NE;
			break;
			case 11:
				setFiring(false);
				startFrame = paulNS::PAUL_STAND_FRAME_NW;
				endFrame = paulNS::PAUL_STAND_FRAME_NW;
				//currentFrame = paulNS::PAUL_STAND_FRAME_NW;
			break;
			case 5:
				setFiring(false);
				startFrame = paulNS::PAUL_STAND_FRAME_SE;
				endFrame = paulNS::PAUL_STAND_FRAME_SE;
				//currentFrame = paulNS::PAUL_STAND_FRAME_SE;
			break;
			case 7:
				setFiring(false);
				startFrame = paulNS::PAUL_STAND_FRAME_SW;
				endFrame = paulNS::PAUL_STAND_FRAME_SW;
				//currentFrame = paulNS::PAUL_STAND_FRAME_SW;
			break;
		}
	}

	if(input->isKeyDown(PAUL_RIGHT_KEY))				 // if move right
	{
		paulshotDir = 90 * (pi / 180);
		shotDir = 3;
		startFrame = paulNS::PAUL_START_FRAME_E;
		endFrame = paulNS::PAUL_END_FRAME_E;
		velocity.x = paulNS::SPEED * frameTime;
		if(input->isKeyDown(PAUL_SHOOT_KEY))		//Shoot E
		{
			setFiring(true);
			startFrame = paulNS::PAUL_SHOOT_START_E;
			endFrame = paulNS::PAUL_SHOOT_END_E;
			velocity.x = 0;
		}
		else
		{
			setFiring(false);
			spriteData.x += paulNS::SPEED * frameTime;
		}
	}
	if(input->isKeyDown(PAUL_LEFT_KEY))             // if move left
	{
		paulshotDir = 270 * (pi / 180);
		shotDir = 9;
		startFrame = paulNS::PAUL_START_FRAME_W;
		endFrame = paulNS::PAUL_END_FRAME_W; 
		velocity.x = -paulNS::SPEED * frameTime;
		if(input->isKeyDown(PAUL_SHOOT_KEY))		//Shoot W
		{
			setFiring(true);
			startFrame = paulNS::PAUL_SHOOT_START_W;
			endFrame = paulNS::PAUL_SHOOT_END_W;
			velocity.x = 0;
		}
		else
		{
			setFiring(false);
			spriteData.x -= paulNS::SPEED * frameTime;
		}
	}
    if(input->isKeyDown(PAUL_UP_KEY))               // if move up
	{
		paulshotDir = 0;
		shotDir = 12;
		startFrame = paulNS::PAUL_START_FRAME_N;
		endFrame = paulNS::PAUL_END_FRAME_N;
		velocity.y = -paulNS::SPEED * frameTime;
		if(input->isKeyDown(PAUL_SHOOT_KEY))		//Shoot N
		{
			setFiring(true);
			startFrame = paulNS::PAUL_SHOOT_START_N;
			endFrame = paulNS::PAUL_SHOOT_END_N;
			velocity.y = 0;
		}
		else
		{
			setFiring(false);
			spriteData.y -= paulNS::SPEED * frameTime;
		}
	}
    if(input->isKeyDown(PAUL_DOWN_KEY))             // if move down
	{
		paulshotDir = 180 * (pi / 180);
		shotDir = 6;
		startFrame = paulNS::PAUL_START_FRAME_S;
		endFrame = paulNS::PAUL_END_FRAME_S;
		velocity.y = paulNS::SPEED * frameTime;
		if(input->isKeyDown(PAUL_SHOOT_KEY))		//Shoot S
		{
			setFiring(true);
			startFrame = paulNS::PAUL_SHOOT_START_S;
			endFrame = paulNS::PAUL_SHOOT_END_S;
			velocity.y = 0;
		}
		else
		{
			setFiring(false);
			spriteData.y += paulNS::SPEED * frameTime;
		}
	}
	if(input->isKeyDown(PAUL_RIGHT_KEY) && input->isKeyDown(PAUL_UP_KEY))				 // if move NE
	{
		paulshotDir = 45 * (pi / 180);
		shotDir = 1;
		startFrame = paulNS::PAUL_START_FRAME_NE;
		endFrame = paulNS::PAUL_END_FRAME_NE;
		velocity.x = paulNS::SPEED/6 * frameTime;
		velocity.y = -paulNS::SPEED/6 * frameTime;
		if(input->isKeyDown(PAUL_SHOOT_KEY))
		{
			setFiring(true);
			startFrame = paulNS::PAUL_SHOOT_START_NE;
			endFrame = paulNS::PAUL_SHOOT_END_NE;
			velocity.x = 0;
			velocity.y = 0;
		}
		else
		{
			setFiring(false);
			spriteData.x += paulNS::SPEED/6 * frameTime;
			spriteData.y -= paulNS::SPEED/6 * frameTime;
		}
	}
	if(input->isKeyDown(PAUL_UP_KEY) && input->isKeyDown(PAUL_LEFT_KEY))             // if move NW
	{
		paulshotDir = 315 * (pi / 180);
		shotDir = 11;
		startFrame = paulNS::PAUL_START_FRAME_NW;
		endFrame = paulNS::PAUL_END_FRAME_NW;
		velocity.x = -paulNS::SPEED/6 * frameTime;
		velocity.y = -paulNS::SPEED/6 * frameTime;
		if(input->isKeyDown(PAUL_SHOOT_KEY))
		{
			setFiring(true);
			startFrame = paulNS::PAUL_SHOOT_START_NW;
			endFrame = paulNS::PAUL_SHOOT_END_NW;
			velocity.x = 0;
			velocity.y = 0;
		}
		else
		{
			setFiring(false);
			spriteData.x -= paulNS::SPEED/6 * frameTime;
			spriteData.y -= paulNS::SPEED/6 * frameTime;
		}
	}
    if(input->isKeyDown(PAUL_DOWN_KEY) && input->isKeyDown(PAUL_RIGHT_KEY))               // if move SE
	{
		paulshotDir = 135 * (pi / 180);
		shotDir = 5;
		startFrame = paulNS::PAUL_START_FRAME_SE;
		endFrame = paulNS::PAUL_END_FRAME_SE;
		velocity.y = paulNS::SPEED/6 * frameTime;
		velocity.x = paulNS::SPEED/6 * frameTime;
		if(input->isKeyDown(PAUL_SHOOT_KEY))
		{
			setFiring(true);
			startFrame = paulNS::PAUL_SHOOT_START_SE;
			endFrame = paulNS::PAUL_SHOOT_END_SE;
			velocity.x = 0;
			velocity.y = 0;
		}
		else
		{
			setFiring(false);
			spriteData.y += paulNS::SPEED/6 * frameTime;
			spriteData.x += paulNS::SPEED/6 * frameTime;
		}
	}
    if(input->isKeyDown(PAUL_DOWN_KEY) && input->isKeyDown(PAUL_LEFT_KEY))             // if move SW
	{
		paulshotDir = 225 * (pi / 180);
		shotDir = 7;
		startFrame = paulNS::PAUL_START_FRAME_SW;
		endFrame = paulNS::PAUL_END_FRAME_SW;
		velocity.y = paulNS::SPEED/6 * frameTime;
		velocity.x = -paulNS::SPEED/6 * frameTime;
		if(input->isKeyDown(PAUL_SHOOT_KEY))
		{
			setFiring(true);
			startFrame = paulNS::PAUL_SHOOT_START_SW;
			endFrame = paulNS::PAUL_SHOOT_END_SW;
			velocity.x = 0;
			velocity.y = 0;
		}
		else
		{
			setFiring(false);
			spriteData.y += paulNS::SPEED/6 * frameTime;
			spriteData.x -= paulNS::SPEED/6 * frameTime;
		}
	}



    Entity::update(frameTime);

}

//=============================================================================
// damage
//=============================================================================
void Paul::damage(WEAPON weapon)
{
    switch(weapon)
    {
    case BULLET:
        //audio->playCue("hit");
        health -= paulNS::BULLET_DAMAGE;
		shieldOn = true;
        break;
    case LASER:
        //audio->playCue("hit");    // play sound
        health -= paulNS::LASER_DAMAGE;
		shieldOn = true;
        break;
	case SHIELD:
        //audio->playCue("hit");    // play sound
        health -= paulNS::SHIELD_DAMAGE;
		shieldOn = true;
        break;
    }
    if (health <= 0)
        dead();
}

//=============================================================================
// dead
//=============================================================================
void Paul::dead()
{
	audio->playCue("explode");
	setdied(true);
    active = false;
    health = 0;
    explosionOn = true;
    shieldOn = false;
    velocity.x = 0.0f;
    velocity.y = 0.0f;
}

//=============================================================================
// revive
//=============================================================================
void Paul::revive()
{
	setdied(false);
	active = true;
    health = 100;
    explosionOn = false;
    shieldOn = false;
    visible = true;
	setCurrentFrame(paulNS::PAUL_STAND_FRAME_E);
}