#include "soldier.h"

const float pi = 3.14159265359f;
static bool keyPress = false;

//=============================================================================
// default constructor
//=============================================================================
Soldier::Soldier() : Entity()
{
    spriteData.width = soldierNS::WIDTH;           // size of soldier
    spriteData.height = soldierNS::HEIGHT;
    spriteData.x = soldierNS::X;                   // location on screen
    spriteData.y = soldierNS::Y;
    spriteData.rect.bottom = soldierNS::HEIGHT;    // rectangle to select parts of an image
    spriteData.rect.right = soldierNS::WIDTH;
    velocity.x = 0;                             // velocity X
    velocity.y = 0;                             // velocity Y
    frameDelay = soldierNS::SOLDIER_ANIMATION_DELAY;
    startFrame = soldierNS::SOLDIER_STAND_FRAME_S;      // SOLDIER starts at frame 149
    endFrame     = soldierNS::SOLDIER_STAND_FRAME_S;      // SOLDIER starts at frame 149
    currentFrame = startFrame;
    radius = soldierNS::WIDTH/2.0;
    shieldOn = false;
	explosionOn = false;
    mass = soldierNS::MASS;
    collisionType = entityNS::CIRCLE;
	solshotDir = 6;
	shotDir = 6;
	loop = true;
}

//=============================================================================
// Initialize the soldier.
// Post: returns true if successful, false if failed
//=============================================================================
bool Soldier::initialize(Game *gamePtr, int width, int height, int ncols,
    TextureManager *textureM)
{
	shield.initialize(gamePtr->getGraphics(), width, height, ncols, textureM);
    shield.setFrames(soldierNS::SHIELD_START_FRAME, soldierNS::SHIELD_END_FRAME);
    shield.setCurrentFrame(soldierNS::SHIELD_START_FRAME);
    shield.setFrameDelay(soldierNS::SHIELD_ANIMATION_DELAY);
    shield.setLoop(false);                  // do not loop animation
	explosion.initialize(gamePtr->getGraphics(), width, height, ncols, textureM);
    explosion.setFrames(soldierNS::EXPLOSION_START_FRAME, soldierNS::EXPLOSION_END_FRAME);
    explosion.setCurrentFrame(soldierNS::EXPLOSION_START_FRAME);
    explosion.setFrameDelay(soldierNS::EXPLOSION_ANIMATION_DELAY);
    explosion.setLoop(false);               // do not loop animation
    return(Entity::initialize(gamePtr, width, height, ncols, textureM));
}

//=============================================================================
// draw soldier
//=============================================================================
void Soldier::draw()
{
	if(explosionOn)
		explosion.draw(spriteData);
	else
	{
		Image::draw();              // draw soldier
		if(shieldOn)
			shield.draw(spriteData, graphicsNS::ALPHA50 & colorFilter);
	}
}

//=============================================================================
// update
// typically called once per frame
// frameTime is used to regulate the speed of movement and animation
//=============================================================================
void Soldier::update(float frameTime)
{
	if(explosionOn)
    {
        explosion.update(frameTime);
        if(explosion.getAnimationComplete())    // if explosion animation complete
        {
            explosionOn = false;                // turn off explosion
            visible = false;
            explosion.setAnimationComplete(false);
            explosion.setCurrentFrame(soldierNS::EXPLOSION_START_FRAME);
        }
    }

	if(shieldOn)
	{
		shield.update(frameTime);
		if(shield.getAnimationComplete())
		{
			shieldOn = false;
			shield.setAnimationComplete(false);
			shield.setCurrentFrame(soldierNS::SHIELD_START_FRAME);
		}
	}

	keyPress = input->anyKeyPressed();

	if(keyPress == false)
	{
		switch (shotDir)
		{
			case 12:
				setFiring(false);
				startFrame = soldierNS::SOLDIER_STAND_FRAME_N;
				endFrame = soldierNS::SOLDIER_STAND_FRAME_N;
				//currentFrame = soldierNS::SOLDIER_STAND_FRAME_N;
			break;
			case 3:
				startFrame = soldierNS::SOLDIER_STAND_FRAME_E;
				endFrame = soldierNS::SOLDIER_STAND_FRAME_E;
				//currentFrame = soldierNS::SOLDIER_STAND_FRAME_E;
			break;
			case 6:
				startFrame = soldierNS::SOLDIER_STAND_FRAME_S;
				endFrame = soldierNS::SOLDIER_STAND_FRAME_S;
				//currentFrame = soldierNS::SOLDIER_STAND_FRAME_S;
			break;
			case 9:
				setFiring(false);
				startFrame = soldierNS::SOLDIER_STAND_FRAME_W;
				endFrame = soldierNS::SOLDIER_STAND_FRAME_W;
				//currentFrame = soldierNS::SOLDIER_STAND_FRAME_W;
			break;
			case 1:
				setFiring(false);
				startFrame = soldierNS::SOLDIER_STAND_FRAME_NE;
				endFrame = soldierNS::SOLDIER_STAND_FRAME_NE;
				//currentFrame = soldierNS::SOLDIER_STAND_FRAME_NE;
			break;
			case 11:
				setFiring(false);
				startFrame = soldierNS::SOLDIER_STAND_FRAME_NW;
				endFrame = soldierNS::SOLDIER_STAND_FRAME_NW;
				//currentFrame = soldierNS::SOLDIER_STAND_FRAME_NW;
			break;
			case 5:
				setFiring(false);
				startFrame = soldierNS::SOLDIER_STAND_FRAME_SE;
				endFrame = soldierNS::SOLDIER_STAND_FRAME_SE;
				//currentFrame = soldierNS::SOLDIER_STAND_FRAME_SE;
			break;
			case 7:
				setFiring(false);
				startFrame = soldierNS::SOLDIER_STAND_FRAME_SW;
				endFrame = soldierNS::SOLDIER_STAND_FRAME_SW;
				//currentFrame = soldierNS::SOLDIER_STAND_FRAME_SW;
			break;
		}
	}

    if(input->isKeyDown(SOLDIER_RIGHT_KEY))				 // if move right
	{
		solshotDir = 90 * (pi / 180);
		shotDir = 3;
		startFrame = soldierNS::SOLDIER_START_FRAME_E;
		endFrame = soldierNS::SOLDIER_END_FRAME_E;
		velocity.x = soldierNS::SPEED * frameTime;
		if(input->isKeyDown(SOLDIER_SHOOT_KEY))		//Shoot E
		{
			setFiring(true);
			startFrame = soldierNS::SOLDIER_SHOOT_START_E;
			endFrame = soldierNS::SOLDIER_SHOOT_END_E;
			velocity.x = 0;
		}
		else
		{
			setFiring(false);
			spriteData.x += soldierNS::SPEED * frameTime;
		}
	}
	if(input->isKeyDown(SOLDIER_LEFT_KEY))             // if move left
	{
		solshotDir = 270 * (pi / 180);
		shotDir = 9;
		startFrame = soldierNS::SOLDIER_START_FRAME_W;
		endFrame = soldierNS::SOLDIER_END_FRAME_W; 
		velocity.x = -soldierNS::SPEED * frameTime;
		if(input->isKeyDown(SOLDIER_SHOOT_KEY))		//Shoot W
		{
			setFiring(true);
			startFrame = soldierNS::SOLDIER_SHOOT_START_W;
			endFrame = soldierNS::SOLDIER_SHOOT_END_W;
			velocity.x = 0;
		}
		else
		{
			setFiring(false);
			spriteData.x -= soldierNS::SPEED * frameTime;
		}
	}
    if(input->isKeyDown(SOLDIER_UP_KEY))               // if move up
	{
		solshotDir = 0;
		shotDir = 12;
		startFrame = soldierNS::SOLDIER_START_FRAME_N;
		endFrame = soldierNS::SOLDIER_END_FRAME_N;
		velocity.y = -soldierNS::SPEED * frameTime;
		if(input->isKeyDown(SOLDIER_SHOOT_KEY))		//Shoot N
		{
			setFiring(true);
			startFrame = soldierNS::SOLDIER_SHOOT_START_N;
			endFrame = soldierNS::SOLDIER_SHOOT_END_N;
			velocity.y = 0;
		}
		else
		{
			setFiring(false);
			spriteData.y -= soldierNS::SPEED * frameTime;
		}
	}
    if(input->isKeyDown(SOLDIER_DOWN_KEY))             // if move down
	{
		solshotDir = 180 * (pi / 180);
		shotDir = 6;
		startFrame = soldierNS::SOLDIER_START_FRAME_S;
		endFrame = soldierNS::SOLDIER_END_FRAME_S;
		velocity.y = soldierNS::SPEED * frameTime;
		if(input->isKeyDown(SOLDIER_SHOOT_KEY))		//Shoot S
		{
			setFiring(true);
			startFrame = soldierNS::SOLDIER_SHOOT_START_S;
			endFrame = soldierNS::SOLDIER_SHOOT_END_S;
			velocity.y = 0;
		}
		else
		{
			setFiring(false);
			spriteData.y += soldierNS::SPEED * frameTime;
		}
	}
	if(input->isKeyDown(SOLDIER_RIGHT_KEY) && input->isKeyDown(SOLDIER_UP_KEY))				 // if move NE
	{
		solshotDir = 45 * (pi / 180);
		shotDir = 1;
		startFrame = soldierNS::SOLDIER_START_FRAME_NE;
		endFrame = soldierNS::SOLDIER_END_FRAME_NE;
		velocity.x = soldierNS::SPEED/6 * frameTime;
		velocity.y = -soldierNS::SPEED/6 * frameTime;
		if(input->isKeyDown(SOLDIER_SHOOT_KEY))
		{
			setFiring(true);
			startFrame = soldierNS::SOLDIER_SHOOT_START_NE;
			endFrame = soldierNS::SOLDIER_SHOOT_END_NE;
			velocity.x = 0;
			velocity.y = 0;
		}
		else
		{
			setFiring(false);
			spriteData.x += soldierNS::SPEED/6 * frameTime;
			spriteData.y -= soldierNS::SPEED/6 * frameTime;
		}
	}
	if(input->isKeyDown(SOLDIER_UP_KEY) && input->isKeyDown(SOLDIER_LEFT_KEY))             // if move NW
	{
		solshotDir = 315 * (pi / 180);
		shotDir = 11;
		startFrame = soldierNS::SOLDIER_START_FRAME_NW;
		endFrame = soldierNS::SOLDIER_END_FRAME_NW;
		velocity.x = -soldierNS::SPEED/6 * frameTime;
		velocity.y = -soldierNS::SPEED/6 * frameTime;
		if(input->isKeyDown(SOLDIER_SHOOT_KEY))
		{
			setFiring(true);
			startFrame = soldierNS::SOLDIER_SHOOT_START_NW;
			endFrame = soldierNS::SOLDIER_SHOOT_END_NW;
			velocity.x = 0;
			velocity.y = 0;
		}
		else
		{
			setFiring(false);
			spriteData.x -= soldierNS::SPEED/6 * frameTime;
			spriteData.y -= soldierNS::SPEED/6 * frameTime;
		}
	}
    if(input->isKeyDown(SOLDIER_DOWN_KEY) && input->isKeyDown(SOLDIER_RIGHT_KEY))               // if move SE
	{
		solshotDir = 135 * (pi / 180);
		shotDir = 5;
		startFrame = soldierNS::SOLDIER_START_FRAME_SE;
		endFrame = soldierNS::SOLDIER_END_FRAME_SE;
		velocity.y = soldierNS::SPEED/6 * frameTime;
		velocity.x = soldierNS::SPEED/6 * frameTime;
		if(input->isKeyDown(SOLDIER_SHOOT_KEY))
		{
			setFiring(true);
			startFrame = soldierNS::SOLDIER_SHOOT_START_SE;
			endFrame = soldierNS::SOLDIER_SHOOT_END_SE;
			velocity.x = 0;
			velocity.y = 0;
		}
		else
		{
			setFiring(false);
			spriteData.y += soldierNS::SPEED/6 * frameTime;
			spriteData.x += soldierNS::SPEED/6 * frameTime;
		}
	}
    if(input->isKeyDown(SOLDIER_DOWN_KEY) && input->isKeyDown(SOLDIER_LEFT_KEY))             // if move SW
	{
		solshotDir = 225 * (pi / 180);
		shotDir = 7;
		startFrame = soldierNS::SOLDIER_START_FRAME_SW;
		endFrame = soldierNS::SOLDIER_END_FRAME_SW;
		velocity.y = soldierNS::SPEED/6 * frameTime;
		velocity.x = -soldierNS::SPEED/6 * frameTime;
		if(input->isKeyDown(SOLDIER_SHOOT_KEY))
		{
			setFiring(true);
			startFrame = soldierNS::SOLDIER_SHOOT_START_SW;
			endFrame = soldierNS::SOLDIER_SHOOT_END_SW;
			velocity.x = 0;
			velocity.y = 0;
		}
		else
		{
			setFiring(false);
			spriteData.y += soldierNS::SPEED/6 * frameTime;
			spriteData.x -= soldierNS::SPEED/6 * frameTime;
		}
	}

	Entity::update(frameTime);
}

//=============================================================================
// damage
//=============================================================================
void Soldier::damage(WEAPON weapon)
{
    switch(weapon)
    {
    case BULLET:
        //audio->playCue("hit");
        health -= soldierNS::BULLET_DAMAGE;
		//shieldOn = true;
        break;
    case LASER:
        //audio->playCue("hit");    // play sound
        health -= soldierNS::LASER_DAMAGE;
		//shieldOn = true;
        break;
	case SHIELD:
        //audio->playCue("hit");    // play sound
        health -= soldierNS::SHIELD_DAMAGE;
		//shieldOn = true;
        break;
    }
    if (health <= 0)
        dead();
	else
		shieldOn = true;
}

//=============================================================================
// dead
//=============================================================================
void Soldier::dead()
{
	audio->playCue("explode");
	setdied(true);
    active = false;
    health = 0;
    explosionOn = true;
    shieldOn = false;
    velocity.x = 0.0f;
    velocity.y = 0.0f;
}

//=============================================================================
// revive
//=============================================================================
void Soldier::revive()
{
	setdied(false);
	active = true;
    health = 100;
    explosionOn = false;
    shieldOn = false;
    visible = true;
	setCurrentFrame(soldierNS::SOLDIER_STAND_FRAME_W);
}