#ifndef _SOLDIER_H
#define _SOLDIER_H
#define WIN32_LEAN_AND_MEAN

#include "entity.h"
#include "constants.h"

namespace soldierNS
{
	const int WIDTH = 96;                   // image width
    const int HEIGHT = 96;                  // image height
    const int X = GAME_WIDTH/2 - WIDTH/2;   // location on screen
    const int Y = GAME_HEIGHT/2 - HEIGHT/2;
    const float ROTATION_RATE = (float)PI/4; // radians per second
    const float SPEED = 100;                // 100 pixels per second
    const float MASS = 300.0f;              // mass
	const int   BULLET_DAMAGE = 10;
	const int	LASER_DAMAGE = 5;
	const int   SHIELD_DAMAGE = 0;
	const int   SHIELD_START_FRAME = 160;    // shield start frame
    const int   SHIELD_END_FRAME = 163;      // shield end frame
    const float SHIELD_ANIMATION_DELAY = 0.1f; // time between frames
	const int   EXPLOSION_START_FRAME = 152; // explosion start frame
    const int   EXPLOSION_END_FRAME = 159;   // explosion end frame
    const float EXPLOSION_ANIMATION_DELAY = 0.1f;   // time between frames
    const int   TEXTURE_COLS = 10;           // texture has 10 columns
    const float SOLDIER_ANIMATION_DELAY = 0.1f;    // time between frames
	const float SOLDIER_SHOOT_DELAY = 0.05f;

	const int   SOLDIER_START_FRAME_E = 0;
	const int   SOLDIER_END_FRAME_E = 7;

	const int   SOLDIER_START_FRAME_N = 8;
	const int   SOLDIER_END_FRAME_N = 15;

	const int   SOLDIER_START_FRAME_NE = 16;
	const int   SOLDIER_END_FRAME_NE = 23;

	const int   SOLDIER_START_FRAME_NW = 24;
	const int   SOLDIER_END_FRAME_NW = 31;

	const int   SOLDIER_START_FRAME_S = 32;
	const int   SOLDIER_END_FRAME_S = 39;

	const int   SOLDIER_START_FRAME_SE = 40;
	const int   SOLDIER_END_FRAME_SE = 47;

	const int   SOLDIER_START_FRAME_SW = 48;
	const int   SOLDIER_END_FRAME_SW = 55;

	const int   SOLDIER_START_FRAME_W = 56;
	const int   SOLDIER_END_FRAME_W = 63;

	const int   SOLDIER_SHOOT_START_E = 67;
	const int   SOLDIER_SHOOT_END_E = 68;

	const int   SOLDIER_SHOOT_START_N = 78;
	const int   SOLDIER_SHOOT_END_N = 79;

	const int   SOLDIER_SHOOT_START_NE = 88;
	const int   SOLDIER_SHOOT_END_NE = 89;

	const int   SOLDIER_SHOOT_START_NW = 98;
	const int   SOLDIER_SHOOT_END_NW = 99;

	const int   SOLDIER_SHOOT_START_S = 108;
	const int   SOLDIER_SHOOT_END_S = 109;

	const int   SOLDIER_SHOOT_START_SE = 116;
	const int   SOLDIER_SHOOT_END_SE = 117;

	const int   SOLDIER_SHOOT_START_SW = 127;
	const int   SOLDIER_SHOOT_END_SW = 128;

	const int   SOLDIER_SHOOT_START_W = 138;
	const int   SOLDIER_SHOOT_END_W = 139;

    const int   SOLDIER_STAND_FRAME_E = 149;      // SOLDIER starts at frame 149
    const int   SOLDIER_STAND_FRAME_W = 145;      // SOLDIER starts at frame 149
	const int   SOLDIER_STAND_FRAME_N = 147;      // SOLDIER starts at frame 149
	const int   SOLDIER_STAND_FRAME_S = 143;      // SOLDIER starts at frame 149
	const int   SOLDIER_STAND_FRAME_SW = 144;      // SOLDIER starts at frame 149
	const int   SOLDIER_STAND_FRAME_NW = 146;      // SOLDIER starts at frame 149
	const int   SOLDIER_STAND_FRAME_NE = 148;      // SOLDIER starts at frame 149
	const int   SOLDIER_STAND_FRAME_SE = 150;      // SOLDIER starts at frame 149
}

class Soldier : public Entity
{
private:
	int   shotDir;
	bool  isdead;
	float solshotDir;
	bool  solfiring;
	bool  explosionOn;
	bool  shieldOn;
	Image shield;
    Image explosion;
public:
	// constructor
	Soldier();

	//inherited member functions
	virtual void draw();
	virtual bool initialize(Game *gamePtr, int width, int height, int ncols,
                            TextureManager *textureM);
    void update(float frameTime);

	void damage(WEAPON);

	void dead();

	void setVelocityX(float v)  {velocity.x = v;}
	void setVelocityY(float v)	{velocity.y = v;}

	void setFiring(bool b) {solfiring = b;}

	float getFiring()	{return solfiring;}

	float getDirection() {return solshotDir;}

	int getSoldierDirection() {return shotDir;}

	bool getShieldOn()  {return shieldOn;}

	void setShieldOn(bool sh)   {shieldOn = sh;}

	void explode();

	void revive();

	bool died() {return isdead;}

	void setdied(bool d) {isdead = d;}

	bool getExplosion() {return explosionOn;}
};

#endif