#include "thesquad.h"
#include <string>
#include <sstream>
#include <cmath>
static int paulScore = 0, stanScore = 0;
using namespace tileCollisionNS;
float pangleDifference;

//=============================================================================
// Constructor
//=============================================================================
TheSquad::TheSquad()
{
	paul.setdied(false);
	soldier[0].setdied(false);
    paulScored = false;
    stanScored = false;
	countDownOn = true;
	countDownTimer = 3;
	menuOn = true;
	roundOver = false;
	mapX = 0;
	mapY = 45;
    zoom = 1.0f;
	paulbulNum = 0;
	paulfireDelay = 0;
	paulreloadDelay = 4;
	solbulNum = 0;
	solfireDelay = 0;
	solreloadDelay = 4;
	for(int i = 0; i < 4; i++)
	{
		canbulNum[i] = 0;
		canfireDelay[i] = 0;
		cannon[i].setFiring(false);
	}
	paulempty = false;
	solempty = false;
	paul.setFiring(false);
	soldier[0].setFiring(false);
	paulhorizontal = 100;
	solhorizontal = 100;
	paulvu = 100;
	solvu = 100;
	dxFontAmmo = new TextDX();
	dxFontHealth = new TextDX();
	dxFontScore = new TextDX();
	dxScore = new TextDX();
	dxSolFontAmmo = new TextDX();
	dxSolFontHealth = new TextDX();
	dxSolFontScore = new TextDX();
	dxSolScore = new TextDX();
}

//=============================================================================
// Destructor
//=============================================================================
TheSquad::~TheSquad()
{
    releaseAll();           // call onLostDevice() for every graphics item
	SAFE_DELETE(dxFontAmmo);
	SAFE_DELETE(dxFontHealth);
	SAFE_DELETE(dxFontScore);
	SAFE_DELETE(dxScore);
}

//=============================================================================
// Initializes the game
// Throws GameError on error
//=============================================================================
void TheSquad::initialize(HWND hwnd)
{
    Game::initialize(hwnd); // throws GameError

	 // 15 pixel high Arial
    if(dxFontAmmo->initialize(graphics, 15, true, false, "Arial") == false)
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing DirectX font"));

	if(dxFontHealth->initialize(graphics, 15, true, false, "Arial") == false)
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing DirectX font"));

	if(dxFontScore->initialize(graphics, 15, true, false, "Arial") == false)
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing DirectX font"));

	if(dxSolFontAmmo->initialize(graphics, 15, true, false, "Arial") == false)
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing DirectX font"));

	if(dxSolFontHealth->initialize(graphics, 15, true, false, "Arial") == false)
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing DirectX font"));

	if(dxSolFontScore->initialize(graphics, 15, true, false, "Arial") == false)
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing DirectX font"));

	// 30 pixel high Arial
    if(dxScore->initialize(graphics, 30, true, false, "Arial") == false)
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing DirectX font"));

	if(dxSolScore->initialize(graphics, 30, true, false, "Arial") == false)
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing DirectX font"));

	// menu texture
    if (!menuTexture.initialize(graphics,MENU_IMAGE))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing menu texture"));

	// tile textures
    if (!tileTextures.initialize(graphics,TILES))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing tile textures"));

    // paul game textures
	if (!paulTextures.initialize(graphics,PAUL_IMAGE))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing game textures"));

	// soldier game textures
	if (!soldierTextures.initialize(graphics,SOLDIER_IMAGE))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing game textures"));

	// cannon game textures
	if (!cannonTextures.initialize(graphics,CANNON_IMAGE))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing game textures"));

	// gauge textures
    if (!dashboardTextures.initialize(graphics, DASHBOARD_TEXTURES))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing dashboard textures"));

	 paulhorizontalBar.initialize(graphics, &dashboardTextures, HBAR_X, HBAR_Y, 1.0f, graphicsNS::GREEN);
	 paulbarGraph.initialize(graphics, &dashboardTextures, BAR_GRAPH_X, BAR_GRAPH_Y, .5f, 25, graphicsNS::GREEN);

	 solhorizontalBar.initialize(graphics, &dashboardTextures, HBAR_X + 780, HBAR_Y, 1.0f, graphicsNS::BLUE);
	 solbarGraph.initialize(graphics, &dashboardTextures, BAR_GRAPH_X + 780, BAR_GRAPH_Y, .5f, 25, graphicsNS::BLUE);

	 // menu image
    if (!menu.initialize(graphics,0,0,0,&menuTexture))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing menu"));

    // tile image
    if (!tile.initialize(graphics,0,0,0,&tileTextures))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing tile"));

	// load map
    mapIn.open(MAP);
    if(!mapIn)
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error loading map data"));
    for(int row=0; row<MAP_HEIGHT; row++)
    {
        for(int col=0; col<MAP_WIDTH; col++)
            mapIn >> tileMap[row][col];
    }
    /*mapX = GAME_HEIGHT;
	mapY = GAME_WIDTH;*/

	// paul
    if (!paul.initialize(this, paulNS::WIDTH, paulNS::HEIGHT, paulNS::TEXTURE_COLS, &paulTextures))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing paul"));
    paul.setFrames(paulNS::PAUL_STAND_FRAME_E, paulNS::PAUL_STAND_FRAME_E);
    paul.setCurrentFrame(paulNS::PAUL_STAND_FRAME_E);
	paul.setMass(paulNS::MASS);
    paul.setX(static_cast<float>(2*(TILE_SIZE)) + mapX*zoom );
    paul.setY(static_cast<float>(4*(TILE_SIZE)) + mapY*zoom );
	paul.setColorFilter(SETCOLOR_ARGB(255,0,100,0));
    //paul.setVelocity(VECTOR2(paulNS::SPEED, paulNS::SPEED)); // VECTOR2(X, Y)

	//paul bullet
	for(int i = 0;i < 25;i++)
	{
		if(!paulbul[i].initialize(this, BulletNS::WIDTH, BulletNS::HEIGHT, BulletNS::TEXTURE_COLS, &paulTextures))
			throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing Bullet sprite"));
		paulbul[i].setScale(.5);
		paulbul[i].setX(BulletNS::X);
		paulbul[i].setY(BulletNS::Y);
	}

	// soldier
    if (!soldier[0].initialize(this, soldierNS::WIDTH, soldierNS::HEIGHT, soldierNS::TEXTURE_COLS, &soldierTextures))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing soldier"));
    soldier[0].setFrames(soldierNS::SOLDIER_STAND_FRAME_W, soldierNS::SOLDIER_STAND_FRAME_W);
    soldier[0].setCurrentFrame(soldierNS::SOLDIER_STAND_FRAME_W);
    soldier[0].setX(static_cast<float>(12*(TILE_SIZE*zoom)) + mapX );
    soldier[0].setY(static_cast<float>(4*(TILE_SIZE*zoom)) + mapY );
	soldier[0].setColorFilter(SETCOLOR_ARGB(255,0,0,150));
 //   //soldier.setVelocity(VECTOR2(soldierNS::SPEED, soldierNS::SPEED)); // VECTOR2(X, Y)

	//soldier bullet
	for(int i = 0;i < 25;i++)
	{
		if(!solbul[i].initialize(this, BulletNS::WIDTH, BulletNS::HEIGHT, BulletNS::TEXTURE_COLS, &soldierTextures))
			throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing Bullet sprite"));
		solbul[i].setScale(.5);
		solbul[i].setX(BulletNS::X - 97);
		solbul[i].setY(BulletNS::Y - 97);
	}

	// cannons
	if (!cannon[0].initialize(this, cannonNS::WIDTH, cannonNS::HEIGHT, cannonNS::TEXTURE_COLS, &cannonTextures))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing cannon"));
    cannon[0].setFrames(cannonNS::CANNON_START_FRAME, cannonNS::CANNON_END_FRAME);
    cannon[0].setCurrentFrame(cannonNS::CANNON_STAND_FRAME_SE);
    cannon[0].setX(static_cast<float>(0.0f*(TILE_SIZE*zoom)) + mapX );
    cannon[0].setY(static_cast<float>(-.25f*(TILE_SIZE*zoom)) + mapY );
	cannon[0].setEdge(CANNON_BOX);
	cannonX[0] = cannon[0].getX();
	cannonY[0] = cannon[0].getY();

	if (!cannon[1].initialize(this, cannonNS::WIDTH, cannonNS::HEIGHT, cannonNS::TEXTURE_COLS, &cannonTextures))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing cannon"));
    cannon[1].setFrames(cannonNS::CANNON_START_FRAME, cannonNS::CANNON_END_FRAME);
    cannon[1].setCurrentFrame(cannonNS::CANNON_STAND_FRAME_SW);
    cannon[1].setX(static_cast<float>(15.0f*(TILE_SIZE*zoom)) + mapX );
    cannon[1].setY(static_cast<float>(-.25f*(TILE_SIZE*zoom)) + mapY );
	cannon[1].setEdge(CANNON_BOX);
	cannonX[1] = cannon[1].getX();
	cannonY[1] = cannon[1].getY();

	if (!cannon[2].initialize(this, cannonNS::WIDTH, cannonNS::HEIGHT, cannonNS::TEXTURE_COLS, &cannonTextures))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing cannon"));
    cannon[2].setFrames(cannonNS::CANNON_START_FRAME, cannonNS::CANNON_END_FRAME);
    cannon[2].setCurrentFrame(cannonNS::CANNON_STAND_FRAME_NE);
    cannon[2].setX(static_cast<float>(0.0f*(TILE_SIZE*zoom)) + mapX );
    cannon[2].setY(static_cast<float>(8.75f*(TILE_SIZE*zoom)) + mapY );
	cannon[2].setEdge(CANNON_BOX);
	cannonX[2] = cannon[2].getX();
	cannonY[2] = cannon[2].getY();

	if (!cannon[3].initialize(this, cannonNS::WIDTH, cannonNS::HEIGHT, cannonNS::TEXTURE_COLS, &cannonTextures))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing cannon"));
    cannon[3].setFrames(cannonNS::CANNON_START_FRAME, cannonNS::CANNON_END_FRAME);
    cannon[3].setCurrentFrame(cannonNS::CANNON_STAND_FRAME_NW);
    cannon[3].setX(static_cast<float>(15.0f*(TILE_SIZE*zoom)) + mapX );
    cannon[3].setY(static_cast<float>(8.75f*(TILE_SIZE*zoom)) + mapY );
	cannon[3].setEdge(CANNON_BOX);
	cannonX[3] = cannon[3].getX();
	cannonY[3] = cannon[3].getY();

	//cannon bullets
	for(int i = 0;i < 4;i++)
		for(int j = 0; j < 50; j++)
	{
		if(!canbul[i][j].initialize(this, BulletNS::LASERW, BulletNS::LASERH, BulletNS::LASER_COLS, &cannonTextures))
			throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing Bullet sprite"));
		canbul[i][j].setCurrentFrame(BulletNS::LASER_FRAME);
		canbul[i][j].setScale(.5);
		canbul[i][j].setX(BulletNS::X - 97);
		canbul[i][j].setY(BulletNS::Y - 97);
	}

	// tileEntity, used for collision testing
    tileEntity.initialize(this,TILE_SIZE,TILE_SIZE,1,&tileTextures);
    tileEntity.setEdge(TILE_EDGE);  // for tile collision
    tileEntity.setCollisionType(entityNS::BOX);
    tileEntity.setMass(TILE_MASS);

    return;
}

//=============================================================================
// Update all game items
//=============================================================================
void TheSquad::update()
{
	if (menuOn)
    {
		countDownTimer = 3;
        if (input->anyKeyPressed())
        {
            menuOn = false;
			for(int i=0; i<4; i++)
				cannon[i].setFiring(false);
            input->clearAll();
        }
    }

	if (paul.died())
	{
		for(int i=0; i<4; i++)
			cannon[i].setFiring(false);
		if(!paul.getExplosion())
		{
			paul.setdied(false);
			stanScore += 1;
			paul.revive();
			roundStart();
		}
	}

	if (soldier[0].died())
	{
		for(int i=0; i<4; i++)
			cannon[i].setFiring(false);
		if(!soldier[0].getExplosion())
		{
			soldier[0].setdied(false);
			paulScore += 1;
			soldier[0].revive();
			roundStart();
		}
	}

	//Weapon Control
	paulfireDelay += frameTime;
	paulreloadDelay += frameTime;
	solfireDelay += frameTime;
	solreloadDelay += frameTime;
	for(int i=0; i<4; i++)
		canfireDelay[i] += frameTime;
		if(paulfireDelay >= 0.15 && paulreloadDelay >= 4 && paul.getFiring())												// allows a shot to be fired while the
		{                                                                                   // stick is held in a direction and ever .25 seconds
			paulbulDir = paul.getDirection();
			paulshotDir = paul.getPaulDirection();
			paulbul[paulbulNum].setX(paul.getCenterX() - 50);			// sets the bullet to come out of the ship
			paulbul[paulbulNum].setY(paul.getCenterY() - 50);
			switch (paulshotDir)
			{
				case 3:
					paulbul[paulbulNum].setX(paul.getX() + 50);
					paulbul[paulbulNum].setY(paul.getY() + 15);
				break;
				case 7:
					paulbul[paulbulNum].setY(paul.getY() + 20);
				break;
				case 6:
					paulbul[paulbulNum].setY(paul.getY() + 50);
					paulbul[paulbulNum].setX(paul.getX() + 15);
				break;
				case 1:
					paulbul[paulbulNum].setX(paul.getX() + 50);
				break;
				case 5:
					paulbul[paulbulNum].setY(paul.getY() + 35);
					paulbul[paulbulNum].setX(paul.getX() + 50);
				break;
				case 9:
					paulbul[paulbulNum].setY(paul.getY() + 10);
				break;
				case 12:
					paulbul[paulbulNum].setX(paul.getX() + 30);
				break;
				case 11:
					paulbul[paulbulNum].setY(paul.getY() - 15);
				break;
			}
			paulbul[paulbulNum].setActive(true);
			paulbul[paulbulNum].setVisible(true);
			paulbul[paulbulNum].setPaulVeloc(paulbulDir, frameTime);	// calls the function that sets the speed and direction of the bullet
			paulfireDelay = 0;								// sets fire delay back to zero 
			paulbul[paulbulNum].update(frameTime);
			if(paulbulNum <= 24)						// allows 10 bullets to be fired at once. if 10 are fired, restores to 0.
			{
				paulvu -= 4;
				paulbulNum += 1;
				audio->playCue("SHOT3");
			}
		}
		if(solfireDelay >= 0.15 && solreloadDelay >= 4 && soldier[0].getFiring())												// allows a shot to be fired while the
		{                                                                                   // stick is held in a direction and ever .25 seconds
			solbulDir = soldier[0].getDirection();
			solshotDir = soldier[0].getSoldierDirection();
			solbul[solbulNum].setX(soldier[0].getCenterX() - 50);			// sets the bullet to come out of the ship
			solbul[solbulNum].setY(soldier[0].getCenterY() - 50);
			switch (solshotDir)
			{
				case 3:
					solbul[solbulNum].setX(soldier[0].getX() + 50);
					solbul[solbulNum].setY(soldier[0].getY() + 15);
				break;
				case 7:
					solbul[solbulNum].setY(soldier[0].getY() + 20);
				break;
				case 6:
					solbul[solbulNum].setY(soldier[0].getY() + 50);
					solbul[solbulNum].setX(soldier[0].getX() + 15);
				break;
				case 1:
					solbul[solbulNum].setX(soldier[0].getX() + 50);
				break;
				case 5:
					solbul[solbulNum].setY(soldier[0].getY() + 35);
					solbul[solbulNum].setX(soldier[0].getX() + 50);
				break;
				case 9:
					solbul[solbulNum].setY(soldier[0].getY() + 10);
				break;
				case 12:
					solbul[solbulNum].setX(soldier[0].getX() + 30);
				break;
				case 11:
					solbul[solbulNum].setY(soldier[0].getY() - 15);
				break;
			}
			solbul[solbulNum].setActive(true);
			solbul[solbulNum].setVisible(true);
			solbul[solbulNum].setSolVeloc(solbulDir, frameTime);	// calls the function that sets the speed and direction of the bullet
			solfireDelay = 0;								// sets fire delay back to zero 
			solbul[solbulNum].update(frameTime);
			if(solbulNum <= 24)						// allows 7 bullets to be fired at once. if 10 are fired, restores to 0.
			{
				solvu -= 4;
				solbulNum += 1;
				audio->playCue("gun1");
			}
		}

	if(input->wasKeyPressed(PAUL_RELOAD_KEY))
	{
		paulbulNum = 0;
		paulempty = true;
		audio->playCue("reload");
		paulreloadDelay = 0;
		paulvu += 100;
	}

	if(input->wasKeyPressed(SOLDIER_RELOAD_KEY))
	{
		solbulNum = 0;
		solempty = true;
		audio->playCue("reload");
		solreloadDelay = 0;
		solvu += 100;
	}

	countDownTimer -= frameTime;
	if(countDownTimer <=0)
	{
		countDownTimer = 0;
		countDownOn = false;
	}
	paulhorizontalBar.set(paulhorizontal);
	paulbarGraph.set(paulvu);
	solhorizontalBar.set(solhorizontal);
	solbarGraph.set(solvu);

	for(int i = 0;i<25;i++)
			paulbul[i].update(frameTime);

	for(int i = 0;i<25;i++)
			solbul[i].update(frameTime);

	for(int j = 0; j < 4; j++)
		for(int i = 0;i<50;i++)
			canbul[j][i].update(frameTime);

	tileEntity.update(frameTime);
	
	soldier[0].update(frameTime);

	paul.update(frameTime);

	for (int i = 0; i < 4; i++)
		cannon[i].update(frameTime);

	for ( int i = 0; i < 4; i++)
	{
		cannon[i].setX(cannonX[i] + mapX);
		cannon[i].setY(cannonY[i] + mapY - 45);
	}
}
//=============================================================================
// Start a new round of play
//=============================================================================
void TheSquad::roundStart()
{
//    // Start players on opposite sides
    paul.setX(static_cast<float>(2*(TILE_SIZE)) + mapX*zoom );
    paul.setY(static_cast<float>(4*(TILE_SIZE)) + mapY*zoom );
    soldier[0].setX(static_cast<float>(12*(TILE_SIZE*zoom)) + mapX );
    soldier[0].setY(static_cast<float>(4*(TILE_SIZE*zoom)) + mapY );
    paul.setVelocity(VECTOR2(0,paulNS::SPEED));
    soldier[0].setVelocity(VECTOR2(0,soldierNS::SPEED));
    paul.setHealth(100);
	soldier[0].setHealth(100);
    paul.revive();
	paulbulNum = 0;
	paulempty = true;
	audio->playCue("reload");
	paulreloadDelay = 0;
	paulvu += 100;
    soldier[0].revive();
	solbulNum = 0;
	solempty = true;
	audio->playCue("reload");
	solreloadDelay = 0;
	solvu += 100;
    countDownTimer = 3;
    countDownOn = true;
    paulScored = false;
    stanScored = false;
	roundOver = false;
	for(int i=0; i<4; i++)
			cannon[i].setFiring(false);
}


//=============================================================================
// Artificial Intelligence
//=============================================================================
void TheSquad::ai()
{
	if(countDownOn)
	{
		for(int i = 0; i < 4; i++)
			cannon[i].setFiring(false);
	}
	else
		for(int i = 0; i < 4; i++)
			cannon[i].setFiring(true);

	for(int i = 0; i < 4; i++)
		cannon[i].setDirection();

	for(int i =0; i<4; i++)
		if(cannon[i].getFiring())
		{
			// NE Corner Cannon
			if(canfireDelay[0] >= 0.20 && cannon[0].getCurrentFrame() > 25 && cannon[0].getCurrentFrame() <= 31)												// allows a shot to be fired while the
				{                                                                                   // stick is held in a direction and ever .25 seconds
					canbulDir[0] = cannon[0].getDirection();
					canbul[0][canbulNum[0]].setX(cannon[0].getCenterX());			// sets the bullet to come out of the ship
					canbul[0][canbulNum[0]].setY(cannon[0].getCenterY());
					canbul[0][canbulNum[0]].setActive(true);
					canbul[0][canbulNum[0]].setVisible(true);
					canbul[0][canbulNum[0]].setCanVeloc(canbulDir[0], frameTime);	// calls the function that sets the speed and direction of the bullet
					canfireDelay[0] = 0;								// sets fire delay back to zero 
					canbul[0][canbulNum[0]].update(frameTime);
					if(canbulNum[0] <= 24)						// allows 10 bullets to be fired at once. if 10 are fired, restores to 0.
					{
						canbulNum[0] += 1;
						audio->playCue("laser-shot");
					}
					else
						canbulNum[0] = 0;
				}

			// NW Corner Cannon
			if(canfireDelay[1] >= 0.20 && cannon[1].getCurrentFrame() > 0 && cannon[1].getCurrentFrame() <= 7)												// allows a shot to be fired while the
				{                                                                                   // stick is held in a direction and ever .25 seconds
					canbulDir[1] = cannon[1].getDirection();
					canbul[1][canbulNum[1]].setX(cannon[1].getCenterX());			// sets the bullet to come out of the ship
					canbul[1][canbulNum[1]].setY(cannon[1].getCenterY());
					canbul[1][canbulNum[1]].setActive(true);
					canbul[1][canbulNum[1]].setVisible(true);
					canbul[1][canbulNum[1]].setCanVeloc(canbulDir[1], frameTime);	// calls the function that sets the speed and direction of the bullet
					canfireDelay[1] = 0;								// sets fire delay back to zero 
					canbul[1][canbulNum[1]].update(frameTime);
					if(canbulNum[1] <= 24)						// allows 10 bullets to be fired at once. if 10 are fired, restores to 0.
					{
						canbulNum[1] += 1;
						audio->playCue("laser-shot");
					}
					else
						canbulNum[1] = 0;
				}

			// SW Corner Cannon
			if(canfireDelay[2] >= 0.20 && cannon[2].getCurrentFrame() > 16 && cannon[2].getCurrentFrame() <= 23)												// allows a shot to be fired while the
				{                                                                                   // stick is held in a direction and ever .25 seconds
					canbulDir[2] = cannon[2].getDirection();
					canbul[2][canbulNum[2]].setX(cannon[2].getCenterX());			// sets the bullet to come out of the ship
					canbul[2][canbulNum[2]].setY(cannon[2].getCenterY());
					canbul[2][canbulNum[2]].setActive(true);
					canbul[2][canbulNum[2]].setVisible(true);
					canbul[2][canbulNum[2]].setCanVeloc(canbulDir[2], frameTime);	// calls the function that sets the speed and direction of the bullet
					canfireDelay[2] = 0;								// sets fire delay back to zero 
					canbul[2][canbulNum[2]].update(frameTime);
					if(canbulNum[2] <= 24)						// allows 10 bullets to be fired at once. if 10 are fired, restores to 0.
					{
						canbulNum[2] += 1;
						audio->playCue("laser-shot");
					}
					else
						canbulNum[2] = 0;
				}

			// SE Corner Cannon
			if(canfireDelay[3] >= 0.20 && cannon[3].getCurrentFrame() > 8 && cannon[3].getCurrentFrame() <= 15)												// allows a shot to be fired while the
				{                                                                                   // stick is held in a direction and ever .25 seconds
					canbulDir[3] = cannon[3].getDirection();
					canbul[3][canbulNum[3]].setX(cannon[3].getCenterX());			// sets the bullet to come out of the ship
					canbul[3][canbulNum[3]].setY(cannon[3].getCenterY());
					canbul[3][canbulNum[3]].setActive(true);
					canbul[3][canbulNum[3]].setVisible(true);
					canbul[3][canbulNum[3]].setCanVeloc(canbulDir[3], frameTime);	// calls the function that sets the speed and direction of the bullet
					canfireDelay[3] = 0;								// sets fire delay back to zero 
					canbul[3][canbulNum[3]].update(frameTime);
					if(canbulNum[3] <= 24)						// allows 10 bullets to be fired at once. if 10 are fired, restores to 0.
					{
						canbulNum[3] += 1;
						audio->playCue("laser-shot");
					}
					else
						canbulNum[3] = 0;
				}
		}
}

//=============================================================================
// Handle collisions
//=============================================================================
void TheSquad::collisions()
{
    VECTOR2 collisionVector;

	// Calculate map grid location of paul
    UINT paulRow = static_cast<UINT>((paul.getY() - mapY + 45)/TILE_SIZE);
    UINT paulCol = static_cast<UINT>((paul.getX() - mapX + 45)/TILE_SIZE);
    // check for collision with left, current, right, bottom left, bottom and bottom right tiles.
    // if tile left of paul may be collided with
    if(paulCol > 0 && paulRow < MAP_HEIGHT && tileMap[paulRow][paulCol-1] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>((paulCol-1)*TILE_SIZE+mapX));    // position tileEntity
        tileEntity.setY(static_cast<float>(paulRow*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between paul and tile
        if(paul.collidesWith(tileEntity, collisionVector))
            paul.bounce(collisionVector,tileEntity);
    }
    // if current tile may be collided with
    if(paulCol >= 0 && paulRow < MAP_HEIGHT && tileMap[paulRow][paulCol] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>(paulCol*TILE_SIZE+mapX));     // position tileEntity
        tileEntity.setY(static_cast<float>(paulRow*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between paul and tile
        if(paul.collidesWith(tileEntity, collisionVector))
            paul.bounce(collisionVector,tileEntity);
    }
    // if tile right of paul may be collided with
    if(paulCol < MAP_WIDTH-1 && paulRow < MAP_HEIGHT && tileMap[paulRow][paulCol+1] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>((paulCol+1)*TILE_SIZE+mapX));   // position tileEntity
        tileEntity.setY(static_cast<float>(paulRow*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between paul and tile
        if(paul.collidesWith(tileEntity, collisionVector))
            paul.bounce(collisionVector,tileEntity);
    }
    // if tile bottom left of paul may be collided with
    if(paulCol > 0 && paulRow < MAP_HEIGHT-1 && tileMap[paulRow+1][paulCol-1] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>((paulCol-1)*TILE_SIZE+mapX));   // position tileEntity
        tileEntity.setY(static_cast<float>((paulRow+1)*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between paul and tile
        if(paul.collidesWith(tileEntity, collisionVector))
            paul.bounce(collisionVector,tileEntity);
    }
    // if tile below paul may be collided with
    if(paulRow < MAP_HEIGHT-1 && tileMap[paulRow+1][paulCol] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>(paulCol*TILE_SIZE+mapX));     // position tileEntity
        tileEntity.setY(static_cast<float>((paulRow+1)*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between paul and tile
        if(paul.collidesWith(tileEntity, collisionVector))
            paul.bounce(collisionVector,tileEntity);
    }
    // if tile bottom right of paul may be collided with
    if(paulCol < MAP_WIDTH-1 && paulRow < MAP_HEIGHT-1 && tileMap[paulRow+1][paulCol+1] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>((paulCol+1)*TILE_SIZE+mapX));   // position tileEntity
        tileEntity.setY(static_cast<float>((paulRow+1)*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between paul and tile
        if(paul.collidesWith(tileEntity, collisionVector))
            paul.bounce(collisionVector,tileEntity);
    }
	 // if tile above paul[0] may be collided with
    if(paulRow > 0 && tileMap[paulRow-1][paulCol] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>(paulCol*TILE_SIZE+mapX));     // position tileEntity
        tileEntity.setY(static_cast<float>((paulRow-1)*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between paul[0] and tile
        if(paul.collidesWith(tileEntity, collisionVector))
            paul.bounce(collisionVector,tileEntity);
    }
	// if tile above left of paul[0] may be collided with
    if(paulCol > 0 && paulRow > 0 && tileMap[paulRow-1][paulCol-1] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>((paulCol-1)*TILE_SIZE+mapX));   // position tileEntity
        tileEntity.setY(static_cast<float>((paulRow-1)*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between paul[0] and tile
        if(paul.collidesWith(tileEntity, collisionVector))
            paul.bounce(collisionVector,tileEntity);
    }
	// if tile above right of paul[0] may be collided with
    if(paulCol < MAP_WIDTH-1 && paulRow > 0 && tileMap[paulRow-1][paulCol+1] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>((paulCol+1)*TILE_SIZE+mapX));   // position tileEntity
        tileEntity.setY(static_cast<float>((paulRow-1)*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between paul[0] and tile
        if(paul.collidesWith(tileEntity, collisionVector))
            paul.bounce(collisionVector,tileEntity);
    }



	// Calculate map grid location of soldier
    UINT soldierRow = static_cast<UINT>((soldier[0].getY() - mapY + 45)/TILE_SIZE);
    UINT soldierCol = static_cast<UINT>((soldier[0].getX() - mapX + 45)/TILE_SIZE);
    // check for collision with left, current, right, bottom left, bottom and bottom right tiles.
    // if tile left of soldier[0] may be collided with
    if(soldierCol > 0 && soldierRow < MAP_HEIGHT && tileMap[soldierRow][soldierCol-1] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>((soldierCol-1)*TILE_SIZE+mapX));    // position tileEntity
        tileEntity.setY(static_cast<float>(soldierRow*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between soldier[0] and tile
        if(soldier[0].collidesWith(tileEntity, collisionVector))
            soldier[0].bounce(collisionVector,tileEntity);
    }
    // if current tile may be collided with
    if(soldierCol >= 0 && soldierRow < MAP_HEIGHT && tileMap[soldierRow][soldierCol] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>(soldierCol*TILE_SIZE+mapX));     // position tileEntity
        tileEntity.setY(static_cast<float>(soldierRow*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between soldier[0] and tile
        if(soldier[0].collidesWith(tileEntity, collisionVector))
            soldier[0].bounce(collisionVector,tileEntity);
    }
    // if tile right of soldier[0] may be collided with
    if(soldierCol < MAP_WIDTH-1 && soldierRow < MAP_HEIGHT && tileMap[soldierRow][soldierCol+1] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>((soldierCol+1)*TILE_SIZE+mapX));   // position tileEntity
        tileEntity.setY(static_cast<float>(soldierRow*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between soldier[0] and tile
        if(soldier[0].collidesWith(tileEntity, collisionVector))
            soldier[0].bounce(collisionVector,tileEntity);
    }
    // if tile bottom left of soldier[0] may be collided with
    if(soldierCol > 0 && soldierRow < MAP_HEIGHT-1 && tileMap[soldierRow+1][soldierCol-1] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>((soldierCol-1)*TILE_SIZE+mapX));   // position tileEntity
        tileEntity.setY(static_cast<float>((soldierRow+1)*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between soldier[0] and tile
        if(soldier[0].collidesWith(tileEntity, collisionVector))
            soldier[0].bounce(collisionVector,tileEntity);
    }
    // if tile below soldier[0] may be collided with
    if(soldierRow < MAP_HEIGHT-1 && tileMap[soldierRow+1][soldierCol] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>(soldierCol*TILE_SIZE+mapX));     // position tileEntity
        tileEntity.setY(static_cast<float>((soldierRow+1)*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between soldier[0] and tile
        if(soldier[0].collidesWith(tileEntity, collisionVector))
            soldier[0].bounce(collisionVector,tileEntity);
    }
    // if tile bottom right of soldier[0] may be collided with
    if(soldierCol < MAP_WIDTH-1 && soldierRow < MAP_HEIGHT-1 && tileMap[soldierRow+1][soldierCol+1] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>((soldierCol+1)*TILE_SIZE+mapX));   // position tileEntity
        tileEntity.setY(static_cast<float>((soldierRow+1)*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between soldier[0] and tile
        if(soldier[0].collidesWith(tileEntity, collisionVector))
            soldier[0].bounce(collisionVector,tileEntity);
    }
	 // if tile above soldier[0] may be collided with
    if(soldierRow > 0 && tileMap[soldierRow-1][soldierCol] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>(soldierCol*TILE_SIZE+mapX));     // position tileEntity
        tileEntity.setY(static_cast<float>((soldierRow-1)*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between soldier[0] and tile
        if(soldier[0].collidesWith(tileEntity, collisionVector))
            soldier[0].bounce(collisionVector,tileEntity);
    }
	// if tile above left of soldier[0] may be collided with
    if(soldierCol > 0 && soldierRow > 0 && tileMap[soldierRow-1][soldierCol-1] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>((soldierCol-1)*TILE_SIZE+mapX));   // position tileEntity
        tileEntity.setY(static_cast<float>((soldierRow-1)*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between soldier[0] and tile
        if(soldier[0].collidesWith(tileEntity, collisionVector))
            soldier[0].bounce(collisionVector,tileEntity);
    }
	// if tile above right of soldier[0] may be collided with
    if(soldierCol < MAP_WIDTH-1 && soldierRow > 0 && tileMap[soldierRow-1][soldierCol+1] >= MAX_COLLISION_TILE)
    {
        tileEntity.setX(static_cast<float>((soldierCol+1)*TILE_SIZE+mapX));   // position tileEntity
        tileEntity.setY(static_cast<float>((soldierRow-1)*TILE_SIZE+mapY));
        tileEntity.setRotatedBoxReady(false);        
        // if collision between soldier[0] and tile
        if(soldier[0].collidesWith(tileEntity, collisionVector))
            soldier[0].bounce(collisionVector,tileEntity);
    }

	if(soldier[0].collidesWith(paul, collisionVector))
	{
		soldier[0].bounce(collisionVector,paul);
		paul.bounce(collisionVector*-1,soldier[0]);
		soldier[0].damage(SHIELD);
		paul.damage(SHIELD);
	}

	for (int i = 0; i < 25; i++)
		for (int j = 0; j < 4; j++)
		{
			if (paulbul[i].collidesWith(cannon[j], collisionVector))
			{
				paulbul[i].setActive(false);
				paulbul[i].setVisible(false);
				//audio->playCue(DESTROYED);
			}
			if (paulbul[i].collidesWith(soldier[0], collisionVector))
			{
				soldier[0].damage(BULLET);
				paulbul[i].setActive(false);
				paulbul[i].setVisible(false);
			}
		}

	for (int i = 0; i < 25; i++)
		for (int j = 0; j < 4; j++)
		{
			if (solbul[i].collidesWith(cannon[j], collisionVector))
			{
				solbul[i].setActive(false);
				solbul[i].setVisible(false);
				//audio->playCue(DESTROYED);
			}
			if (solbul[i].collidesWith(paul, collisionVector))
			{
				paul.damage(BULLET);
				solbul[i].setActive(false);
				solbul[i].setVisible(false);
			}
		}
    
	for (int i = 0; i < 4; i ++)
		for (int j = 0; j < 50; j++)
		{
			if(canbul[i][j].collidesWith(paul, collisionVector))
			{
				paul.damage(LASER);
				canbul[i][j].setVisible(false);
				canbul[i][j].setActive(false);
			}
			if(canbul[i][j].collidesWith(soldier[0], collisionVector))
			{
				soldier[0].damage(LASER);
				canbul[i][j].setVisible(false);
				canbul[i][j].setActive(false);
			}
		}
	
}

//=============================================================================
// Render game items
//=============================================================================
void TheSquad::render()
{
    graphics->spriteBegin();                // begin drawing sprites

	for(int row=0; row<MAP_HEIGHT; row++)       // for each row of map
    {
        //tile.setY( (float)(row*TILE_SIZE*zoom) ); // set tile Y
        for(int col=0; col<MAP_WIDTH; col++)    // for each column of map
        {
            if(tileMap[row][col] >= 0)          // if tile present
            {
                tile.setX( (float)(col*(TILE_SIZE*zoom)) + mapX*zoom );  // set tile X
                tile.setY( (float)(row*(TILE_SIZE*zoom)) + mapY*zoom );  // set tile Y
                // if tile on screen
                if( (tile.getX() > -TILE_SIZE && tile.getX() < GAME_WIDTH) &&
                    (tile.getY() > -TILE_SIZE && tile.getY() < GAME_HEIGHT) )
                    tile.draw(UINT(tileMap[row][col]));             // draw tile
            }
        }
    }

	paul.draw();
	for(int i = 0; i < 25; i++)
		paulbul[i].draw();

	soldier[0].draw();
	for(int i = 0; i < 25; i++)
		solbul[i].draw();

	for(int i = 0; i < 4; i++)
		cannon[i].draw();

	for(int j=0; j<4; j++)
		for(int i = 0; i < 50; i++)
			canbul[j][i].draw();

	paulhorizontalBar.set(paul.getHealth());
	paulhorizontalBar.draw(graphicsNS::FILTER);
	paulbarGraph.draw(graphicsNS::FILTER);
	solhorizontalBar.set(soldier[0].getHealth());
	solhorizontalBar.draw(graphicsNS::FILTER);
	solbarGraph.draw(graphicsNS::FILTER);

	std::string paulString = static_cast<std::ostringstream*>( &(std::ostringstream() << paulScore) )->str();	//USE FOR GAME SCORE
	std::string solString = static_cast<std::ostringstream*>( &(std::ostringstream() << stanScore) )->str();	//USE FOR GAME SCORE
	dxFontAmmo->setFontColor(graphicsNS::GREEN);
	dxFontAmmo->print("Ammo",40,35);

	dxFontHealth->setFontColor(graphicsNS::GREEN);
	dxFontHealth->print("Shields",140,35);

	dxFontScore->setFontColor(graphicsNS::GREEN);
	dxFontScore->print("Score",200,35);

	dxScore->setFontColor(graphicsNS::GREEN);
	dxScore->print(paulString,208,4);

	dxSolFontAmmo->setFontColor(graphicsNS::BLUE);
	dxSolFontAmmo->print("Ammo",820,35);

	dxSolFontHealth->setFontColor(graphicsNS::BLUE);
	dxSolFontHealth->print("Shields",920,35);

	dxSolFontScore->setFontColor(graphicsNS::BLUE);
	dxSolFontScore->print("Score",980,35);

	dxSolScore->setFontColor(graphicsNS::BLUE);
	dxSolScore->print(solString,990,4);

	if(menuOn)
        menu.draw();

    graphics->spriteEnd();                  // end drawing sprites
}

//=============================================================================
// The graphics device was lost.
// Release all reserved video memory so graphics device may be reset.
//=============================================================================
void TheSquad::releaseAll()
{
	menuTexture.onLostDevice();
	tileTextures.onLostDevice();
	paulTextures.onLostDevice();
	soldierTextures.onLostDevice();
	dashboardTextures.onLostDevice();
	cannonTextures.onLostDevice();
	dxFontAmmo->onLostDevice();
	dxFontHealth->onLostDevice();
	dxFontScore->onLostDevice();
	dxScore->onLostDevice();
	dxSolFontAmmo->onLostDevice();
	dxSolFontHealth->onLostDevice();
	dxSolFontScore->onLostDevice();
	dxSolScore->onLostDevice();
    Game::releaseAll();
    return;
}

//=============================================================================
// The grahics device has been reset.
// Recreate all surfaces.
//=============================================================================
void TheSquad::resetAll()
{
	menuTexture.onResetDevice();
	tileTextures.onResetDevice();
	paulTextures.onResetDevice();
	soldierTextures.onResetDevice();
	dashboardTextures.onResetDevice();
	cannonTextures.onResetDevice();
	dxFontAmmo->onResetDevice();
	dxFontHealth->onResetDevice();
	dxFontScore->onResetDevice();
	dxScore->onResetDevice();
	dxSolFontAmmo->onResetDevice();
	dxSolFontHealth->onResetDevice();
	dxSolFontScore->onResetDevice();
	dxSolScore->onResetDevice();
    Game::resetAll();
    return;
}
