#ifndef _THESQUAD_H             // Prevent multiple definitions if this 
#define _THESQUAD_H             // file is included in more than one place
#define WIN32_LEAN_AND_MEAN

#include <string>
#include <fstream>
#include "game.h"
#include "textureManager.h"
#include "image.h"
#include "paul.h"
#include "soldier.h"
#include "bullet.h"
#include "cannon.h"
#include "dashboard.h"
#include "textDX.h"

const int HBAR_X = 125;
const int HBAR_Y = -14;
const int BAR_GRAPH_X = 2;
const int BAR_GRAPH_Y = 2;

namespace tileCollisionNS
{
    const char FONT[] = "Arial Bold";  // font
	const COLOR_ARGB FONT_COLOR = graphicsNS::YELLOW;
	const int TEXTURE_SIZE = 96;
    const int TILE_SIZE = 64;
    const int MAP_HEIGHT = 10;
    const int MAP_WIDTH = 16;
    const float SCROLL_RATE = 10;
    const float MAX_PAUL_SPEED = 20;
    const USHORT EMPTY = 0;
    const USHORT MAX_COLLISION_TILE = 8;
    const int TILE_EDGE_TOP = -TILE_SIZE/2;  // for tile collision
    const int TILE_EDGE_BOTTOM = TILE_SIZE/2;
    const int TILE_EDGE_LEFT = -TILE_SIZE/2;
    const int TILE_EDGE_RIGHT = TILE_SIZE/2;
    const RECT TILE_EDGE = {TILE_EDGE_LEFT, TILE_EDGE_TOP, TILE_EDGE_RIGHT, TILE_EDGE_BOTTOM};
    const float TILE_MASS = 1.0f;
}

//=============================================================================
// This class is the core of the game
//=============================================================================
class TheSquad : public Game
{
private:
    // game items
    TextureManager menuTexture;   // menu texture
    TextureManager gameTextures;    // game texture
	TextureManager paulTextures;
	TextureManager tileTextures;        // tile textures
	TextureManager soldierTextures;
	TextureManager dashboardTextures;
	TextureManager cannonTextures;
	TextDX	*dxFontAmmo;
	TextDX  *dxFontHealth;
	TextDX	*dxFontScore;
	TextDX	*dxScore;
	TextDX	*dxSolFontAmmo;
	TextDX  *dxSolFontHealth;
	TextDX	*dxSolFontScore;
	TextDX	*dxSolScore;
	Entity  tileEntity;                 // for tile collision
	Paul	paul;
	Soldier soldier[10];
	Cannon	cannon[4];
	Image   tile;
	Image	menu;
	Bullet	paulbul[30], solbul[30], canbul[4][50];
	Bar		paulhorizontalBar, solhorizontalBar;
	BarGraph paulbarGraph, solbarGraph;
	std::ifstream mapIn;                // used to read map
	float   roundTimer;         // time until new round starts
	bool    paulScored, stanScored;
	bool menuOn;
	bool roundOver;
	bool    countDownOn;
	float   countDownTimer;
	float   mapX, mapY;
	float	cannonX[4], cannonY[4], cannonDir[4];
    float   zoom;
	int paulbulNum, solbulNum, canbulNum[4], solshotDir, paulshotDir;
	float paulfireDelay, paulbulDir, paulreloadDelay,
		  solfireDelay, solbulDir, solreloadDelay,
		  canfireDelay[4], canbulDir[4];
	float paulhorizontal, paulvu,
		  solhorizontal, solvu;
	double shotAngle;
	bool paulempty, solempty;
	// Tile numbers
    // 0 empty, 1-17 land, 50 water
    USHORT tileMap[tileCollisionNS::MAP_HEIGHT][tileCollisionNS::MAP_WIDTH];

public:
    // Constructor
    TheSquad();

    // Destructor
    virtual ~TheSquad();

    // Initialize the game
    void initialize(HWND hwnd);
    void update();      // must override pure virtual from Game
    void ai();          // "
    void collisions();  // "
    void render();      // "
	void roundStart();  // start a new round of play
    void releaseAll();
    void resetAll();
};

#endif
